function cfg = singleplotTFR(cfg,data)
%
% --------------------------------------------------------------------------
% SingePlotTFR plots the time-frequency representation of a single channel
% using the output of FREQANALYSIS or WAVELETANALYSIS.
%
%
% Use as
%   SinglePlotTFR(cfg,data)
%    
% data is produced by FREQANALYSIS or WAVELETANALYSIS.
% cfg structure contains all configurations details for the plotting
%
% cfg.baseline        = 'yes','no' or [time1 time2] (default = 'no')  
% cfg.baselinetype    = 'absolute' or 'relative' (default = 'absolute')   
% cfg.xlim            = 'maxmin' or [xmin xmax] (default = 'maxmin')
% cfg.ylim            = 'maxmin' or [ymin ymax] (default = 'maxmin')
% cfg.zlim            = 'maxmin','absmax' or [zmin zmax] (default = 'maxmin')
% cfg.channelindex        = channel number (not label)
% cfg.channelname         = channel label
%
% See freqbaseline on details/optiond on baselining


clf

if ~isfield(cfg,'baseline'),        cfg.baseline   = 'no'; end;
if ~isfield(cfg,'baselinetype'),    cfg.baselinetype = 'absolute'; end;
if ~isfield(cfg,'xlim'),            cfg.xlim       = 'maxmin'; end;
if ~isfield(cfg,'ylim'),            cfg.ylim       = 'maxmin'; end;
if ~isfield(cfg,'zlim'),            cfg.zlim       = 'maxmin'; end;


if isfield(cfg,'channelindex') && isfield(cfg,'channelname')
    warning('You have defined both cfg.channelindex and cfg.channelname; cfg.channelname will be used');
    cfg = rmfield(cfg,'channelindex');
end

if ~isfield(cfg,'channelindex') && ~isfield(cfg,'channelname')
    warning ('channelname chose automatically');
    cfg.channelindex = 1; 
end


if isfield(cfg,'channelname')
    cfg.channel = channelselection(cfg.channelname,data.label);
    if isempty(cfg.channel)
        error(['Channel ' cfg.channelname ' not found']);
    else
        cfg.channelindex =  match_str(data.label, cfg.channelname);
    end
end

if ~strcmp(cfg.baseline,'no')
    if strcmp(cfg.baseline,'yes') 
        cfg.baseline = [];
        cfg.baseline(1) = min(data.toi);
        cfg.baseline(2) = 0;          
    end
    data = freqbaseline(cfg,data);
end

TFR = data.powspctrm;

if strcmp(cfg.xlim,'maxmin')
    xmin = min(data.toi);
    xmax = max(data.toi);   
else
    xmin = cfg.xlim(1); 
    xmax = cfg.xlim(2); 
end


if strcmp(cfg.ylim,'maxmin')
    ymin = min(data.foi);
    ymax = max(data.foi);   
else
    ymin = cfg.ylim(1); 
    ymax = cfg.ylim(2); 
end


if strcmp(cfg.zlim,'maxmin')
    zmin = min(min(min(TFR(cfg.channelindex,:,:))));
    zmax = max(max(max(TFR(cfg.channelindex,:,:))));
elseif strcmp(cfg.zlim,'absmax')
    mval = max([abs(min(min(min(TFR(cfg.channelindex,:,:))))) abs(max(max(max(TFR(cfg.channelindex,:,:))))) ]); 
    zmin = - mval;
    zmax = mval;
else
    zmin = cfg.zlim(1); 
    zmax = cfg.zlim(2); 
end


xidc = find(data.toi >= xmin & data.toi <= xmax);
yidc = find(data.foi >= ymin & data.foi <= ymax);

data.toi = data.toi(xidc);
data.foi =  data.foi(yidc);
TFR = TFR(:,yidc,xidc);
TFR = shiftdim(TFR,1);
imagesc(data.toi, data.foi, TFR(:,:,cfg.channelindex), [zmin,zmax]);axis xy;colorbar;


if ~isfield(cfg,'channelname')
    cfg.channelname = data.label{cfg.channelindex};
end
    
title([cfg.channelname ' / ' num2str(cfg.channelindex)])
